"""
This example shows how to create buttons that have pixelpefect collisions.
"""
import gui, pygame, pixelperfect

class ourMouse:
    #we create our own mouse object because it has to have a rect and a hitbox.
    def __init__(self):
        self.image = pygame.Surface((1,1))#create image - used for getting the rect
        self.rect = self.image.get_rect()#create rect
        self.rect.centerx, self.rect.centery = pygame.mouse.get_pos()#place rect at mouses position
        self.hitmask = pygame.surfarray.array2d(self.image)
        #create hitmask, we use 'array2d' because the mouse doesnt have alpha or a mask
        #because its only one square pixel in size
        #so we use the faster array2d to just get an array of the image
        self.hitmask[0][0] = 1#finally set the hitmask so that it isnt 'off'
        #if we simply left the hitmask as it was, it would have seemed to pixelperfect like it was not hitting our button,
        #no matter whether it actually does or not.
        #So we set the hitmask to '1', indicating that its only pixel can collide with our button

class ourButton(gui.Button):
    def __init__(self, centerx, centery):
        gui.Button.__init__(self, centerx, centery)

    def pp_clicked(self, mouse):
        #here is where we run pixelperfect to see if we hit the button
        self.hitmask = pygame.surfarray.array_alpha(self.message_surf)
        #we create a hitmask
        #because our button is text only we use the font surface in the button to determine the hitmask
        #if it had an image we could create an array based on its image like this:
        #   self.hitmask = pygame.surfarray.array_colorkey(self.image)
        # we use array_alpha because pygame text surfaces  use alpha, not bitmasks to make them transparent.
        if mouse == True:
            pass
        else:
            return False

        obj = ourMouse()
        a = pixelperfect.collide(self, obj)
        return a

    def highlight(self):
        #over-write the buttons highlight to use our pixelperfect system
        self.hitmask = pygame.surfarray.array_alpha(self.message_surf)
        obj = ourMouse()
        a = pixelperfect.collide(self, obj)
        #here we check if the button's highlighting has been toggled off
        #self.t_high_l indicates whether the button can highlight
        if self.t_high_l == True:
            if a:
                #if ourMouse did collide with ou button we highlight
                
                #self.do_hl indicates to ht ebutton that it should run the highlight script
                #because the mouse is hovering over it.
                self.do_hl = True
            else:
                self.do_hl = False
        else:
            self.do_hl = False
        
        
def main():
    #first set up all of the pygame stuff
    pygame.init()
    screen = pygame.display.set_mode((550, 650))
    pygame.display.set_caption("GUI 1")
    screen.fill([0,0,0])

    #now we create our button we use our new button class instead of the gui's,
    #that way we have access to the new features we added.
    exit_button = ourButton(200, 100)
    #except we make it really big, so it is easier to click
    exit_button.Set_Font(font=pygame.font.Font(None,100))

    while 1:

        #and our loop is almost completely the same except that
        #we call exit_button.pp_clicked instead of exit_button.clicked
        m_c = gui.mouse_down(1)
        if exit_button.pp_clicked(m_c) == True:
            pygame.quit()
            return 0
        screen.fill([0,0,0])
        exit_button.draw(screen, 'exit')

        pygame.display.flip()
        m_c = 0


main()
