import os, pygame, sys, random, pickle, string
from pygame.locals import *

def load_image(name, colorkey=None):
    """loads an image into memory"""
    fullname = os.path.join('data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()


class Isometric_Tile_Map:
    """The class that collects, holds, returns, and places all map data"""
    def __init__(self, globex, globey, tile_size="default", map_width="default", map_height="default"):
        """self,screen, globex, globey, tile_size=default, map_width=default, map_height=default"""
        self.globex = globex
        self.globey = globey

        self.map_data = []

        self.set_params(tile_size, map_width, map_height)
        
        self.tiles = []
        self.images = []
        self.tile_pos = []

        self.map_surf = None
        self.sub_map_surf = None
        self.first_time_draw = True

    def set_params(self, tile_size, map_width, map_height):
        self.tile_size_default = 100
        self.tile_size = 0
        if tile_size == "default":
            self.tile_size = self.tile_size_default
        else:
            self.tile_size = tile_size

        self.map_size_default = 20,20
        self.map_size = 0
        if (map_width, map_height) == ("default", "default"):
            self.map_size = self.map_size_default
        else:
            self.map_size = map_width, map_height

    def load_tiles(self,letters,images,transparency):
        for i in range(len(letters)):
            a = load_image(images[i], -1)
            if transparency[i] is not 0:
                a[0].set_alpha(transparency[i])
            else:
                pass
            self.tiles.append(letters[i])
            self.images.append(a)
        
    def move_globe(self,globex,globey):
        self.globex = globex
        self.globey = globey
        
    def put_tile_at_grid_height(self,numx,numy,height):
        half = self.tile_size/2
        quart = self.tile_size/4

        h_ny = half*numy
        
        tilex = numx * half -(h_ny)-self.globex
        tiley = numx * quart + (h_ny - quart)-(quart*numy)-self.globey-(quart*height)
        tile = [tilex+250,tiley]
        return tile
    
    def place_tile(self, screen, kind, locs, x_kind, y_kind):
        full = self.tile_size
        half = self.tile_size/2
        quart = self.tile_size/4
        h_q = half+quart
        for i in range(len(self.tiles)):
            if self.tiles[i] == kind:
                if locs[0] <= screen.get_width() and locs[0] >= 0-full:
                    if locs[1] <= screen.get_height() and locs[1] >= 0-quart:
                        screen.blit(self.images[i][0], locs, (0,0,full,h_q))
                        self.tile_pos.append([self.tiles[i], locs[0], locs[1]])
                        if self.images[i][0].get_alpha is not None:
                            if x_kind == "end" or x_kind == "-":
                                screen.blit(self.images[i][0], (locs[0]+half,locs[1]), (full,0,half,h_q))
                            if y_kind ==  "end" or y_kind == "-":
                                screen.blit(self.images[i][0], (locs[0],locs[1]), (full+half,0,half,h_q))
            else:
                pass

    def load_data(self, filename):
        n_test=open(os.path.join("plates",filename))
        n_test=n_test.read()
        a = string.split(n_test)
        found_map = False
        map_col = 0
        map_data = []
        all_map_data = []
        peak = 0
        line=0
        on_peak=1
        for i in range(len(a)):
            if a[i] == "peak:":
                peak=int(float(a[i+1]))
                line+=1
            if found_map == False:
                t=peak-(peak-on_peak)
                if a[i] == "/map_data_height_%s"%t:
                    found_map = True
                    line+=1
                    continue
                    
            if found_map == True:
                map_data.append(a[i])
                
                if not a[i+1] == "\end_map":
                    map_col+=1
                    
                else:
                    found_map = False
                    all_map_data.append(map_data)
                    map_data = []
                    on_peak+=1

        self.map_data = all_map_data
                    

    def draw_map(self, screen):
        for isoz in range(len(self.map_data)):
            for rows in xrange(self.map_size[0]):
                for cols in xrange(self.map_size[1]):
                    try:
                        x_kind = self.map_data[isoz][rows][cols+1]
                    except:
                        x_kind = "end"
                    try:
                        y_kind = self.map_data[isoz][rows+1][cols]
                    except:
                        y_kind = "end"
                    self.place_tile(screen,(self.map_data[isoz][rows][cols]),
                                    self.put_tile_at_grid_height(cols,rows,isoz),
                                    x_kind,
                                    y_kind)

        return screen
